
from Globals import InitializeClass
from AccessControl import ClassSecurityInfo

from Products.CMFCore.permissions import View
from Products.CMFCore.permissions import ModifyPortalContent
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFCore.CMFCatalogAware import CMFCatalogAware
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
from Products.CMFExtFile.cacheable import Cacheable
from Products.ExtFile import ExtFile

try:
    from zope.interface import implements
    from Products.CMFExtFile.interfaces import ICMFExtFile
    from Products.GenericSetup.interfaces import IDAVAware
except ImportError:
    implements = None

try:
    from zope.component.factory import Factory
except ImportError:
    Factory = None


factory_type_information = (
  { 'id'             : 'ExtFile'
  , 'title'          : 'ExtFile'
  , 'meta_type'      : 'Portal ExtFile'
  , 'description'    : """\
ExtFile objects can contain arbitrary downloadable files.
"""
  , 'icon'           : 'file_icon.gif'
  , 'product'        : 'CMFExtFile'
  , 'factory'        : 'addExtFile'
  , 'immediate_view' : 'metadata_edit_form'
  , 'aliases'        : {'(Default)': 'index_html',
                        'view': 'extfile_view',
                        'edit': 'file_edit_form',
                        'metadata': 'metadata_edit_form'}
  , 'actions'        : ( { 'id'            : 'view'
                         , 'name'          : 'View'
                         , 'title'         : 'View'
                         , 'action': 'string:${object_url}/view'
                         , 'permissions'   : (View,)
                         }
                       , { 'id'            : 'download'
                         , 'name'          : 'Download'
                         , 'title'         : 'Download'
                         , 'action': 'string:${object/static_url}'
                         , 'permissions'   : (View,)
                         }
                       , { 'id'            : 'edit'
                         , 'name'          : 'Edit'
                         , 'title'         : 'Edit'
                         , 'action': 'string:${object_url}/edit'
                         , 'permissions'   : (ModifyPortalContent,)
                         }
                       , { 'id'            : 'metadata'
                         , 'name'          : 'Metadata'
                         , 'title'         : 'Metadata'
                         , 'action': 'string:${object_url}/metadata'
                         , 'permissions'   : (ModifyPortalContent,)
                         }
                       )
  }
,
)


def addExtFile( self
           , id=''
           , title=''
           , file=''
           , content_type=''
           , precondition=''
           , subject=()
           , description=''
           , contributors=()
           , effective_date=None
           , expiration_date=None
           , format=None
           , language=''
           , rights=''
           ):
    """
    Add an ExtFile
    """

    if not id and getattr(file, 'filename', None) is not None:
        # Generate id from filename and make sure it has no 'bad' chars
        id = file.filename
        id = id[max(id.rfind('/'),
                    id.rfind('\\'),
                    id.rfind(':'))+1:]
        title = title or id
        id = ExtFile.normalize_id(id)

    self = self.this()

    # Instantiate the object and set its description.
    fobj = CMFExtFile( id, title, '', content_type, precondition, subject
               , description, contributors, effective_date, expiration_date
               , format, language, rights
               )

    # Add the File instance to self
    self._setObject(id, fobj)

    # 'Upload' the file.  This is done now rather than in the
    # constructor because the object is now in the ZODB and
    # can span ZODB objects.
    if file != '':
        self._getOb(id).manage_upload(file, content_type)

    return id


class CMFExtFile( PortalContent
          , Cacheable
          , ExtFile.ExtFile
          , DefaultDublinCoreImpl
          ):
    """
        A Portal-managed ExtFile
    """

    if implements is not None:
        implements(ICMFExtFile, IDAVAware)

    __implements__ = ( PortalContent.__implements__
                     , DefaultDublinCoreImpl.__implements__
                     )

    meta_type = 'Portal ExtFile'
    effective_date = expiration_date = None
    _isDiscussable = 1
    precondition = ''

    manage_options = (
        ExtFile.ExtFile.manage_options[:3] +
        PortalContent.manage_options[:1] +
        CMFCatalogAware.manage_options +
        ExtFile.ExtFile.manage_options[3:]
    )

    security = ClassSecurityInfo()

    def __init__( self
                , id
                , title=''
                , file=''
                , content_type=''
                , precondition=''
                , subject=()
                , description=''
                , contributors=()
                , effective_date=None
                , expiration_date=None
                , format=None
                , language='en-US'
                , rights=''
                ):
        ExtFile.ExtFile.__init__( self, id, title, description
                               , permission_check=0
                               , redirect_default_view=0 )

        # If no file format has been passed in, rely on what OFS.Image.File
        # detected. Unlike Images, which have code to try and pick the content
        # type out of the binary data, File objects only provide the correct
        # type if a "hint" in the form of a filename extension is given.
        if format is None:
            format = self.content_type

        DefaultDublinCoreImpl.__init__( self, title, subject, description
                               , contributors, effective_date, expiration_date
                               , format, language, rights )

    security.declareProtected(View, 'SearchableText')
    def SearchableText(self):
        """
        SearchableText is used for full text searches of a portal.
        Returns Title and Description of the file.
        """
        return "%s %s" % (self.title, self.description)

    security.declarePrivate('manage_afterAdd')
    def manage_afterAdd(self, item, container):
        """Both of my parents have an afterAdd method"""
        ExtFile.ExtFile.manage_afterAdd(self, item, container)
        PortalContent.manage_afterAdd(self, item, container)

    security.declarePrivate('manage_afterClone')
    def manage_afterClone(self, item):
        """Both of my parents have an afterClone method"""
        ExtFile.ExtFile.manage_afterClone(self, item)
        PortalContent.manage_afterClone(self, item)

    security.declarePrivate('manage_beforeDelete')
    def manage_beforeDelete(self, item, container):
        """Both of my parents have a beforeDelete method"""
        PortalContent.manage_beforeDelete(self, item, container)
        ExtFile.ExtFile.manage_beforeDelete(self, item, container)

    security.declarePrivate('_isNotEmpty')
    def _isNotEmpty(self, file):
        """ Do various checks on 'file' to try to determine non emptiness. """
        if not file:
            return 0                    # Catches None, Missing.Value, ''
        elif file and (type(file) is type('')):
            return 1
        elif getattr(file, 'filename', None):
            return 1
        elif not getattr(file, 'read', None):
            return 0
        else:
            file.seek(0,2)              # 0 bytes back from end of file
            t = file.tell()             # Report the location
            file.seek(0)                # and return pointer back to 0
            if t: return 1
            else: return 0

    security.declarePrivate('_edit')
    def _edit(self, precondition='', file=''):
        """ Perform changes for user """
        if precondition: self.precondition = precondition
        elif self.precondition: del self.precondition

        if self._isNotEmpty(file):
            self.manage_upload(file)

    security.declareProtected(ModifyPortalContent, 'edit')
    def edit(self, precondition='', file=''):
        """ Update and reindex. """
        self._edit( precondition, file )
        self.reindexObject()

    security.declareProtected(View, 'Format')
    def Format(self):
        """ Dublin Core element - resource format """
        return self.content_type

    security.declareProtected(ModifyPortalContent, 'setFormat')
    def setFormat(self, format):
        """ Dublin Core element - resource format """
        self.manage_changeProperties(content_type=format)

    security.declareProtected(ModifyPortalContent, 'PUT')
    def PUT(self, REQUEST, RESPONSE):
        """ Handle HTTP (and FTP) PUT requests """
        ExtFile.ExtFile.PUT( self, REQUEST, RESPONSE )
        self.reindexObject()


InitializeClass(CMFExtFile)

if Factory is not None:
    ExtFileFactory = Factory(CMFExtFile)

