
from AccessControl import ClassSecurityInfo
from Globals import InitializeClass
from Globals import DTMLFile

from Products.CMFCore.permissions import ViewManagementScreens
from Products.CMFCore.permissions import ManageProperties

from Products.CMFCore.utils import ImmutableId
from Products.CMFCore.utils import SimpleItemWithProperties

from OFS.Cache import Cacheable


class ExtFileTool(ImmutableId, SimpleItemWithProperties):
    """Holds settings for CMF ExtFiles and CMF ExtImages"""

    id = 'extfile_properties'
    meta_type = 'CMF ExtFile Tool'

    security = ClassSecurityInfo()

    _properties = (
        {'id': 'cache_manager', 'type': 'selection', 'mode': 'w', 'label': 'Cache objects using',
         'select_variable': 'getCacheManagerIds'},
        {'id': 'create_previews', 'type': 'boolean', 'mode': 'w', 'label': 'Create previews'},
        {'id': 'preview_maxx', 'type': 'int', 'mode': 'w', 'label': 'Preview max width'},
        {'id': 'preview_maxy', 'type': 'int', 'mode': 'w', 'label': 'Preview max height'},
        {'id': 'preview_ratio', 'type': 'boolean', 'mode': 'w', 'label': 'Keep aspect ratio'},
    )

    cache_manager = ''
    create_previews = True
    preview_maxx = 350
    preview_maxy = 350
    preview_ratio = True

    manage_options = (
        SimpleItemWithProperties.manage_options[:1] +
        ({'label': 'Overview', 'action': 'manage_overview'},) +
        SimpleItemWithProperties.manage_options[1:]
    )

    security.declareProtected(ViewManagementScreens, 'manage_overview')
    manage_overview = DTMLFile('www/explainExtFileTool', globals())

    security.declareProtected(ViewManagementScreens, 'getCacheManagerIds')
    def getCacheManagerIds(self):
        """Return ids of available cache managers."""
        managers = ['(None)']
        for m in Cacheable.ZCacheable_getManagerIds.im_func(self):
            managers.append(m['id'])
        return tuple(managers)

    security.declareProtected(ManageProperties, 'manage_editProperties')
    def manage_editProperties(self, REQUEST):
        """Edit object properties via the web.

        The purpose of this method is to change all property values,
        even those not listed in REQUEST; otherwise checkboxes that
        get turned off will be ignored. Use manage_changeProperties()
        instead for most situations.
        """
        if REQUEST.get('cache_manager', '') == '(None)':
            REQUEST.set('cache_manager', '')
        return SimpleItemWithProperties.manage_editProperties(self, REQUEST)


InitializeClass(ExtFileTool)

