#
# CMFExtFile tests
#

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Products.CMFExtFile.tests import CMFExtFileTestCase
from Products.CMFCore.permissions import ModifyPortalContent

gifImage = CMFExtFileTestCase.gifImage
notImage = CMFExtFileTestCase.notImage
makeFileUpload = CMFExtFileTestCase.makeFileUpload

default_user = CMFExtFileTestCase.default_user
default_password = CMFExtFileTestCase.default_password
auth_info = '%s:%s' % (default_user, default_password)


class TestExtFileAdd(CMFExtFileTestCase.FunctionalTestCase):

    def testAddFileFromString(self):
        # Add ExtFile from string
        file = open(gifImage, 'rb').read()
        self.folder.invokeFactory('ExtFile', id='file', file=file)
        self.assertEqual(self.folder.file.Format(), 'application/octet-stream')
        self.assertEqual(self.folder.file.get_size(), self._fsize(gifImage))

    def testAddFileFromFileHandle(self):
        # Add ExtFile from file handle
        file = open(gifImage, 'rb')
        self.folder.invokeFactory('ExtFile', id='file', file=file)
        self.assertEqual(self.folder.file.Format(), 'application/octet-stream')
        self.assertEqual(self.folder.file.get_size(), self._fsize(gifImage))

    def testAddFileFromFileUpload(self):
        # Add ExtFile from file upload
        file = makeFileUpload(gifImage, 'image/gif')
        self.folder.invokeFactory('ExtFile', id='file', file=file)
        self.assertEqual(self.folder.file.Format(), 'image/gif')
        self.assertEqual(self.folder.file.get_size(), self._fsize(gifImage))

    def testAddFileFromStringWithFormat(self):
        # Add ExtFile from file string passing a content_type
        file = open(gifImage, 'rb').read()
        self.folder.invokeFactory('ExtFile', id='file', file=file, format='image/gif')
        self.assertEqual(self.folder.file.Format(), 'image/gif')
        self.assertEqual(self.folder.file.get_size(), self._fsize(gifImage))

    def testAddFileFromFileHandleWithFormat(self):
        # Add ExtFile from file handle passing a content_type
        file = open(gifImage, 'rb')
        self.folder.invokeFactory('ExtFile', id='file', file=file, format='image/gif')
        self.assertEqual(self.folder.file.Format(), 'image/gif')
        self.assertEqual(self.folder.file.get_size(), self._fsize(gifImage))

    def testAddFileWithIdFromFilename(self):
        # Add ExtFile without id - should use filename instead
        file = makeFileUpload(gifImage, 'image/gif', filename='foo.gif')
        self.folder.invokeFactory('ExtFile', id='', file=file)
        self.assertEqual(self.folder['foo.gif'].Format(), 'image/gif')
        self.assertEqual(self.folder['foo.gif'].get_size(), self._fsize(gifImage))

    def testPUTAddFile(self):
        # FTP PUT an ExtFile
        folder_path = self.folder.absolute_url(1)
        response = self.publish(folder_path+'/file',
                                stdin=open(notImage, 'rb'),
                                env={'CONTENT_TYPE': 'application/octet-stream'},
                                request_method='PUT',
                                basic=auth_info)
        self.assertEqual(response.getStatus(), 201)
        self.assertEqual(self.folder.file.Format(), 'application/octet-stream')
        self.assertEqual(self.folder.file.get_size(), self._fsize(notImage))


class TestExtFileEdit(CMFExtFileTestCase.FunctionalTestCase):

    def afterSetUp(self):
        CMFExtFileTestCase.FunctionalTestCase.afterSetUp(self)
        self.folder.invokeFactory('ExtFile', id='file')
        self.assertEqual(self.folder.file.Format(), '')
        self.assertEqual(self.folder.file.get_size(), 0)

    def testEditFileFromString(self):
        # Edit file passing string
        file = open(gifImage, 'rb').read()
        self.folder.file.edit(file=file)
        self.assertEqual(self.folder.file.Format(), 'application/octet-stream')
        self.assertEqual(self.folder.file.get_size(), self._fsize(gifImage))

    def testEditFileFromFileHandle(self):
        # Edit file passing file handle
        file = open(gifImage, 'rb')
        self.folder.file.edit(file=file)
        self.assertEqual(self.folder.file.Format(), 'application/octet-stream')
        self.assertEqual(self.folder.file.get_size(), self._fsize(gifImage))

    def testEditFileFromFileUpload(self):
        # Edit file passing FileUpload instance
        # NB: Must provide filename here or FileUpload evaluates to false
        file = makeFileUpload(gifImage, 'image/gif', filename='foo')
        self.folder.file.edit(file=file)
        self.assertEqual(self.folder.file.Format(), 'image/gif')
        self.assertEqual(self.folder.file.get_size(), self._fsize(gifImage))

    def testPUTEditFile(self):
        # FTP PUT an ExtFile
        folder_path = self.folder.absolute_url(1)
        response = self.publish(folder_path+'/file',
                                stdin=open(gifImage, 'rb'),
                                env={'CONTENT_TYPE': 'image/gif'},
                                request_method='PUT',
                                basic=auth_info)
        self.assertEqual(response.getStatus(), 204)
        self.assertEqual(self.folder.file.Format(), 'image/gif')
        self.assertEqual(self.folder.file.get_size(), self._fsize(gifImage))


class TestExtFileCataloging(CMFExtFileTestCase.FunctionalTestCase):

    def afterSetUp(self):
        CMFExtFileTestCase.FunctionalTestCase.afterSetUp(self)
        self.catalog = self.portal.portal_catalog
        self.catalog.addColumn('Format')
        self.workflow = self.portal.portal_workflow
        self.workflow.setChainForPortalTypes(('ExtFile',), ())

    def testAddFileFromFileString(self):
        # Add ExtFile from string
        self.folder.invokeFactory('ExtFile', id='file', title='Foo',
                                  file=open(gifImage, 'rb').read())
        brain = self.catalog(getId='file')[0]
        self.assertEqual(brain.Format, 'application/octet-stream')
        self.assertEqual(brain.Title, 'Foo')

    def testAddFileFromFileHandle(self):
        # Add ExtFile from file handle
        self.folder.invokeFactory('ExtFile', id='file', title='Foo',
                                  file=open(gifImage, 'rb'))
        brain = self.catalog(getId='file')[0]
        self.assertEqual(brain.Format, 'application/octet-stream')
        self.assertEqual(brain.Title, 'Foo')

    def testAddFileFromFileUpload(self):
        # Add ExtFile from file upload
        self.folder.invokeFactory('ExtFile', id='file', title='Foo',
                                  file=makeFileUpload(gifImage, 'image/gif'))
        brain = self.catalog(getId='file')[0]
        self.assertEqual(brain.Format, 'image/gif')
        self.assertEqual(brain.Title, 'Foo')

    def testAddFileFromFileHandleWithFormat(self):
        # Add ExtFile from file handle passing a content_type
        self.folder.invokeFactory('ExtFile', id='file', title='Foo',
                                  file=open(gifImage, 'rb'), format='image/gif')
        brain = self.catalog(getId='file')[0]
        self.assertEqual(brain.Format, 'image/gif')
        self.assertEqual(brain.Title, 'Foo')

    def testAddFileWithIdFromFilename(self):
        # Add ExtFile without id - should use filename instead
        self.folder.invokeFactory('ExtFile', id='', title='Foo',
                                  file=makeFileUpload(gifImage, 'image/gif', filename='foo.gif'))
        brain = self.catalog(getId='foo.gif')[0]
        self.assertEqual(brain.Format, 'image/gif')
        self.assertEqual(brain.Title, 'Foo')

    def testPUTAddFile(self):
        # FTP PUT an ExtFile
        folder_path = self.folder.absolute_url(1)
        response = self.publish(folder_path+'/file',
                                stdin=open(notImage, 'rb'),
                                env={'CONTENT_TYPE': 'application/octet-stream'},
                                request_method='PUT',
                                basic=auth_info)
        self.assertEqual(response.getStatus(), 201)
        brain = self.catalog(getId='file')[0]
        self.assertEqual(brain.Format, 'application/octet-stream')
        self.assertEqual(brain.Title, '')

    def testEditFileFromString(self):
        # Edit file passing string
        self.folder.invokeFactory('ExtFile', id='file')
        self.folder.file.edit(file=open(gifImage, 'rb').read())
        brain = self.catalog(getId='file')[0]
        self.assertEqual(brain.Format, 'application/octet-stream')

    def testEditFileFromFileHandle(self):
        # Edit file passing file handle
        self.folder.invokeFactory('ExtFile', id='file')
        self.folder.file.edit(file=open(gifImage, 'rb'))
        brain = self.catalog(getId='file')[0]
        self.assertEqual(brain.Format, 'application/octet-stream')

    def testEditFileFromFileUpload(self):
        # Edit file passing FileUpload instance
        self.folder.invokeFactory('ExtFile', id='file')
        # NB: Must provide filename here or FileUpload evaluates to false
        self.folder.file.edit(file=makeFileUpload(gifImage, 'image/gif', filename='foo'))
        brain = self.catalog(getId='file')[0]
        self.assertEqual(brain.Format, 'image/gif')

    def testPUTEditFile(self):
        # FTP PUT an ExtFile
        self.folder.invokeFactory('ExtFile', id='file')

        # We don't have workflow here so we do this by hand
        self.folder.file.manage_permission(ModifyPortalContent, ('Manager', 'Owner'))

        folder_path = self.folder.absolute_url(1)
        response = self.publish(folder_path+'/file',
                                stdin=open(gifImage, 'rb'),
                                env={'CONTENT_TYPE': 'image/gif'},
                                request_method='PUT',
                                basic=auth_info)
        self.assertEqual(response.getStatus(), 204)
        brain = self.catalog(getId='file')[0]
        self.assertEqual(brain.Format, 'image/gif')
        self.assertEqual(brain.Title, '')


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestExtFileAdd))
    suite.addTest(makeSuite(TestExtFileEdit))
    suite.addTest(makeSuite(TestExtFileCataloging))
    return suite

if __name__ == '__main__':
    framework()

