#
# CMFExtImage tests
#

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Products.CMFExtFile.tests import CMFExtFileTestCase
from Products.CMFCore.permissions import ModifyPortalContent

gifImage = CMFExtFileTestCase.gifImage
jpegImage = CMFExtFileTestCase.jpegImage
tiffImage = CMFExtFileTestCase.tiffImage
makeFileUpload = CMFExtFileTestCase.makeFileUpload

default_user = CMFExtFileTestCase.default_user
default_password = CMFExtFileTestCase.default_password
auth_info = '%s:%s' % (default_user, default_password)


class TestExtImageAdd(CMFExtFileTestCase.FunctionalTestCase):

    def testAddImageFromString(self):
        # Add ExtImage from string
        image = open(gifImage, 'rb').read()
        self.folder.invokeFactory('ExtImage', id='image', file=image)
        self.assertEqual(self.folder.image.Format(), 'image/gif')
        self.assertEqual(self.folder.image.get_size(), self._fsize(gifImage))
        self.failUnless(self.folder.image.has_preview)

    def testAddImageFromFileHandle(self):
        # Add ExtImage from file handle
        image = open(gifImage, 'rb')
        self.folder.invokeFactory('ExtImage', id='image', file=image)
        self.assertEqual(self.folder.image.Format(), 'image/gif')
        self.assertEqual(self.folder.image.get_size(), self._fsize(gifImage))
        self.failUnless(self.folder.image.has_preview)

    def testAddImageFromFileUpload(self):
        # Add ExtImage from file upload
        image = makeFileUpload(gifImage, 'image/gif')
        self.folder.invokeFactory('ExtImage', id='image', file=image)
        self.assertEqual(self.folder.image.Format(), 'image/gif')
        self.assertEqual(self.folder.image.get_size(), self._fsize(gifImage))
        self.failUnless(self.folder.image.has_preview)

    def testAddImageFromStringWithFormat(self):
        # Add ExtImage from file string passing a content_type
        image = open(tiffImage, 'rb').read()
        self.folder.invokeFactory('ExtImage', id='image', file=image, format='image/tiff')
        self.assertEqual(self.folder.image.Format(), 'image/tiff')
        self.assertEqual(self.folder.image.get_size(), self._fsize(tiffImage))
        self.failUnless(self.folder.image.has_preview)

    def testAddImageFromStringWithFormatKnownContent(self):
        # Content type detection by content trumps passed-in content type
        image = open(gifImage, 'rb').read()
        self.folder.invokeFactory('ExtImage', id='image', file=image, format='image/jpeg')
        self.assertEqual(self.folder.image.Format(), 'image/gif')
        self.assertEqual(self.folder.image.get_size(), self._fsize(gifImage))
        self.failUnless(self.folder.image.has_preview)

    def testAddImageFromFileHandleWithFormat(self):
        # Add ExtImage from file handle passing a content_type
        image = open(tiffImage, 'rb')
        self.folder.invokeFactory('ExtImage', id='image', file=image, format='image/tiff')
        self.assertEqual(self.folder.image.Format(), 'image/tiff')
        self.assertEqual(self.folder.image.get_size(), self._fsize(tiffImage))
        self.failUnless(self.folder.image.has_preview)

    def testAddImageFromFileHandleWithFormatKnownContent(self):
        # Content type detection by content trumps passed-in content type
        image = open(gifImage, 'rb')
        self.folder.invokeFactory('ExtImage', id='image', file=image, format='image/jpeg')
        self.assertEqual(self.folder.image.Format(), 'image/gif')
        self.assertEqual(self.folder.image.get_size(), self._fsize(gifImage))
        self.failUnless(self.folder.image.has_preview)

    def testAddImageWithIdFromFilename(self):
        # Add ExtImage without id - should use filename instead
        image = makeFileUpload(gifImage, 'image/gif', filename='foo.gif')
        self.folder.invokeFactory('ExtImage', id='', file=image)
        self.assertEqual(self.folder['foo.gif'].Format(), 'image/gif')
        self.assertEqual(self.folder['foo.gif'].get_size(), self._fsize(gifImage))
        self.failUnless(self.folder['foo.gif'].has_preview)

    def testPUTAddImage(self):
        # FTP PUT an ExtImage
        folder_path = self.folder.absolute_url(1)
        response = self.publish(folder_path+'/image',
                                stdin=open(gifImage, 'rb'),
                                env={'CONTENT_TYPE': 'image/gif'},
                                request_method='PUT',
                                basic=auth_info)
        self.assertEqual(response.getStatus(), 201)
        self.assertEqual(self.folder.image.Format(), 'image/gif')
        self.assertEqual(self.folder.image.get_size(), self._fsize(gifImage))


class TestExtImageEdit(CMFExtFileTestCase.FunctionalTestCase):

    def afterSetUp(self):
        CMFExtFileTestCase.FunctionalTestCase.afterSetUp(self)
        self.folder.invokeFactory('ExtImage', id='image')
        self.assertEqual(self.folder.image.Format(), '')
        self.assertEqual(self.folder.image.get_size(), 0)
        self.failIf(self.folder.image.has_preview)

    def testEditImageFromString(self):
        # Edit image passing string
        image = open(gifImage, 'rb').read()
        self.folder.image.edit(file=image)
        self.assertEqual(self.folder.image.Format(), 'image/gif')
        self.assertEqual(self.folder.image.get_size(), self._fsize(gifImage))
        self.failUnless(self.folder.image.has_preview)

    def testEditImageFromFileHandle(self):
        # Edit image passing file handle
        image = open(gifImage, 'rb')
        self.folder.image.edit(file=image)
        self.assertEqual(self.folder.image.Format(), 'image/gif')
        self.assertEqual(self.folder.image.get_size(), self._fsize(gifImage))
        self.failUnless(self.folder.image.has_preview)

    def testEditImageFromFileUpload(self):
        # Edit image passing FileUpload instance
        # NB: Must provide filename here or FileUpload evaluates to false
        image = makeFileUpload(gifImage, 'image/gif', filename='foo')
        self.folder.image.edit(file=image)
        self.assertEqual(self.folder.image.Format(), 'image/gif')
        self.assertEqual(self.folder.image.get_size(), self._fsize(gifImage))
        self.failUnless(self.folder.image.has_preview)

    def testPUTEditImage(self):
        # FTP PUT an ExtImage
        folder_path = self.folder.absolute_url(1)
        response = self.publish(folder_path+'/image',
                                stdin=open(gifImage, 'rb'),
                                env={'CONTENT_TYPE': 'image/gif'},
                                request_method='PUT',
                                basic=auth_info)
        self.assertEqual(response.getStatus(), 204)
        self.assertEqual(self.folder.image.Format(), 'image/gif')
        self.assertEqual(self.folder.image.get_size(), self._fsize(gifImage))


class TestExtImageCataloging(CMFExtFileTestCase.FunctionalTestCase):

    def afterSetUp(self):
        CMFExtFileTestCase.FunctionalTestCase.afterSetUp(self)
        self.catalog = self.portal.portal_catalog
        self.catalog.addColumn('Format')
        self.workflow = self.portal.portal_workflow
        self.workflow.setChainForPortalTypes(('ExtImage',), ())

    def testAddImageFromFileString(self):
        # Add ExtImage from string
        self.folder.invokeFactory('ExtImage', id='image', title='Foo',
                                  file=open(gifImage, 'rb').read())
        brain = self.catalog(getId='image')[0]
        self.assertEqual(brain.Format, 'image/gif')
        self.assertEqual(brain.Title, 'Foo')

    def testAddImageFromFileHandle(self):
        # Add ExtImage from file handle
        self.folder.invokeFactory('ExtImage', id='image', title='Foo',
                                  file=open(gifImage, 'rb'))
        brain = self.catalog(getId='image')[0]
        self.assertEqual(brain.Format, 'image/gif')
        self.assertEqual(brain.Title, 'Foo')

    def testAddImageFromFileUpload(self):
        # Add ExtImage from file upload
        self.folder.invokeFactory('ExtImage', id='image', title='Foo',
                                  file=makeFileUpload(gifImage, 'image/gif'))
        brain = self.catalog(getId='image')[0]
        self.assertEqual(brain.Format, 'image/gif')
        self.assertEqual(brain.Title, 'Foo')

    def testAddImageFromFileHandleWithFormat(self):
        # Add ExtImage from file handle passing a content_type
        self.folder.invokeFactory('ExtImage', id='image', title='Foo',
                                  file=open(tiffImage, 'rb'), format='image/tiff')
        brain = self.catalog(getId='image')[0]
        self.assertEqual(brain.Format, 'image/tiff')
        self.assertEqual(brain.Title, 'Foo')

    def testAddImageFromFileHandleWithFormatKnownContent(self):
        # Content type detection by content trumps passed-in content type
        self.folder.invokeFactory('ExtImage', id='image', title='Foo',
                                  file=open(gifImage, 'rb'), format='image/jpeg')
        brain = self.catalog(getId='image')[0]
        self.assertEqual(brain.Format, 'image/gif')
        self.assertEqual(brain.Title, 'Foo')

    def testAddImageWithIdFromFilename(self):
        # Add ExtImage without id - should use filename instead
        self.folder.invokeFactory('ExtImage', id='', title='Foo',
                                  file=makeFileUpload(gifImage, 'image/gif', filename='foo.gif'))
        brain = self.catalog(getId='foo.gif')[0]
        self.assertEqual(brain.Format, 'image/gif')
        self.assertEqual(brain.Title, 'Foo')

    def testPUTAddImage(self):
        # FTP PUT an ExtImage
        folder_path = self.folder.absolute_url(1)
        response = self.publish(folder_path+'/image',
                                stdin=open(gifImage, 'rb'),
                                env={'CONTENT_TYPE': 'image/gif'},
                                request_method='PUT',
                                basic=auth_info)
        self.assertEqual(response.getStatus(), 201)
        brain = self.catalog(getId='image')[0]
        self.assertEqual(brain.Format, 'image/gif')
        self.assertEqual(brain.Title, '')

    def testEditImageFromString(self):
        # Edit image passing string
        self.folder.invokeFactory('ExtImage', id='image')
        self.folder.image.edit(file=open(gifImage, 'rb').read())
        brain = self.catalog(getId='image')[0]
        self.assertEqual(brain.Format, 'image/gif')

    def testEditImageFromFileHandle(self):
        # Edit image passing file handle
        self.folder.invokeFactory('ExtImage', id='image')
        self.folder.image.edit(file=open(gifImage, 'rb'))
        brain = self.catalog(getId='image')[0]
        self.assertEqual(brain.Format, 'image/gif')

    def testEditImageFromFileUpload(self):
        # Edit image passing FileUpload instance
        self.folder.invokeFactory('ExtImage', id='image')
        # NB: Must provide filename here or FileUpload evaluates to false
        self.folder.image.edit(file=makeFileUpload(gifImage, 'image/gif', filename='foo'))
        brain = self.catalog(getId='image')[0]
        self.assertEqual(brain.Format, 'image/gif')

    def testPUTEditImage(self):
        # FTP PUT an ExtImage
        self.folder.invokeFactory('ExtImage', id='image')

        # We don't have workflow here so we do this by hand
        self.folder.image.manage_permission(ModifyPortalContent, ('Manager', 'Owner'))

        folder_path = self.folder.absolute_url(1)
        response = self.publish(folder_path+'/image',
                                stdin=open(gifImage, 'rb'),
                                env={'CONTENT_TYPE': 'image/gif'},
                                request_method='PUT',
                                basic=auth_info)
        self.assertEqual(response.getStatus(), 204)
        brain = self.catalog(getId='image')[0]
        self.assertEqual(brain.Format, 'image/gif')
        self.assertEqual(brain.Title, '')


class TestExtImagePreviews(CMFExtFileTestCase.CMFExtFileTestCase):

    def afterSetUp(self):
        CMFExtFileTestCase.CMFExtFileTestCase.afterSetUp(self)
        self.props = self.portal.extfile_properties

    def testCreatePreviewModeOn(self):
        self.props.manage_changeProperties(create_previews=True)
        self.folder.invokeFactory('ExtImage', id='image', file=open(jpegImage, 'rb'))
        self.failUnless(self.folder.image.has_preview)

    def testCreatePreviewModeOff(self):
        self.props.manage_changeProperties(create_previews=False)
        self.folder.invokeFactory('ExtImage', id='image', file=open(jpegImage, 'rb'))
        self.failIf(self.folder.image.has_preview)

    def testCreatePreviewMaxX(self):
        self.props.manage_changeProperties(preview_maxx=25)
        self.folder.invokeFactory('ExtImage', id='image', file=open(jpegImage, 'rb'))
        self.failUnless(self.folder.image.has_preview)
        self.assertEqual(self.folder.image.prev_width(), 25)

    def testCreatePreviewMaxY(self):
        self.props.manage_changeProperties(preview_maxy=25)
        self.folder.invokeFactory('ExtImage', id='image', file=open(jpegImage, 'rb'))
        self.failUnless(self.folder.image.has_preview)
        self.assertEqual(self.folder.image.prev_height(), 25)

    def testCreatePreviewRatioOn(self):
        self.props.manage_changeProperties(preview_ratio=True,
                                           preview_maxx=25, preview_maxy=25)
        self.folder.invokeFactory('ExtImage', id='image', file=open(jpegImage, 'rb'))
        self.failUnless(self.folder.image.has_preview)
        self.assertEqual(self.folder.image.prev_width(), 25)
        self.assertEqual(self.folder.image.prev_height(), 16)

    def testCreatePreviewRatioOff(self):
        self.props.manage_changeProperties(preview_ratio=False,
                                           preview_maxx=25, preview_maxy=25)
        self.folder.invokeFactory('ExtImage', id='image', file=open(jpegImage, 'rb'))
        self.failUnless(self.folder.image.has_preview)
        self.assertEqual(self.folder.image.prev_width(), 25)
        self.assertEqual(self.folder.image.prev_height(), 25)

    def testEditPreviewModeOn(self):
        self.props.manage_changeProperties(create_previews=True)
        self.folder.invokeFactory('ExtImage', id='image')
        self.folder.image.edit(file=open(jpegImage, 'rb'))
        self.failUnless(self.folder.image.has_preview)

    def testEditPreviewModeOff(self):
        self.props.manage_changeProperties(create_previews=False)
        self.folder.invokeFactory('ExtImage', id='image')
        self.folder.image.edit(file=open(jpegImage, 'rb'))
        self.failIf(self.folder.image.has_preview)

    def testEditPreviewMaxX(self):
        self.props.manage_changeProperties(preview_maxx=25)
        self.folder.invokeFactory('ExtImage', id='image')
        self.folder.image.edit(file=open(jpegImage, 'rb'))
        self.failUnless(self.folder.image.has_preview)
        self.assertEqual(self.folder.image.prev_width(), 25)

    def testEditPreviewMaxY(self):
        self.props.manage_changeProperties(preview_maxy=25)
        self.folder.invokeFactory('ExtImage', id='image')
        self.folder.image.edit(file=open(jpegImage, 'rb'))
        self.failUnless(self.folder.image.has_preview)
        self.assertEqual(self.folder.image.prev_height(), 25)

    def testEditPreviewRatioOn(self):
        self.props.manage_changeProperties(preview_ratio=True,
                                           preview_maxx=25, preview_maxy=25)
        self.folder.invokeFactory('ExtImage', id='image')
        self.folder.image.edit(file=open(jpegImage, 'rb'))
        self.failUnless(self.folder.image.has_preview)
        self.assertEqual(self.folder.image.prev_width(), 25)
        self.assertEqual(self.folder.image.prev_height(), 16)

    def testEditPreviewRatioOff(self):
        self.props.manage_changeProperties(preview_ratio=False,
                                           preview_maxx=25, preview_maxy=25)
        self.folder.invokeFactory('ExtImage', id='image')
        self.folder.image.edit(file=open(jpegImage, 'rb'))
        self.failUnless(self.folder.image.has_preview)
        self.assertEqual(self.folder.image.prev_width(), 25)
        self.assertEqual(self.folder.image.prev_height(), 25)

    def testEditDeletesPreview(self):
        self.props.manage_changeProperties(preview_maxx=25, preview_maxy=25)
        self.folder.invokeFactory('ExtImage', id='image', file=open(jpegImage, 'rb'))
        self.failUnless(self.folder.image.has_preview)
        self.assertEqual(self.folder.image.prev_width(), 25)
        self.assertEqual(self.folder.image.prev_height(), 16)
        # Turn off preview creation, the next edit will delete the preview.
        self.props.manage_changeProperties(create_previews=False)
        self.folder.image.edit(file=open(tiffImage, 'rb'))
        self.failIf(self.folder.image.has_preview)

    def testPUTRetainsPreviewBoundaries(self):
        self.props.manage_changeProperties(preview_maxx=25, preview_maxy=25)
        self.folder.invokeFactory('ExtImage', id='image', file=open(jpegImage, 'rb'))
        self.failUnless(self.folder.image.has_preview)
        self.assertEqual(self.folder.image.prev_width(), 25)
        self.assertEqual(self.folder.image.prev_height(), 16)
        request = self.app.REQUEST
        request['BODYFILE'] = open(tiffImage, 'rb')
        self.folder.image.PUT(request, request.RESPONSE)
        self.assertEqual(self.folder.image.prev_width(), 25)
        self.assertEqual(self.folder.image.prev_height(), 17)

    def testPUTDeletesPreview(self):
        self.props.manage_changeProperties(preview_maxx=25, preview_maxy=25)
        self.folder.invokeFactory('ExtImage', id='image', file=open(jpegImage, 'rb'))
        self.failUnless(self.folder.image.has_preview)
        self.assertEqual(self.folder.image.prev_width(), 25)
        self.assertEqual(self.folder.image.prev_height(), 16)
        # Turn off preview creation, the next PUT will delete the preview.
        self.props.manage_changeProperties(create_previews=False)
        request = self.app.REQUEST
        request['BODYFILE'] = open(tiffImage, 'rb')
        self.folder.image.PUT(request, request.RESPONSE)
        self.failIf(self.folder.image.has_preview)

    #def testManageHTTPUploadDoesNotAffectPreview(self):
    #    self.props.manage_changeProperties(preview_maxx=25, preview_maxy=25)
    #    self.folder.invokeFactory('ExtImage', id='image', file=open(jpegImage, 'rb'))
    #    self.failUnless(self.folder.image.has_preview)
    #    self.assertEqual(self.folder.image.prev_width(), 25)
    #    self.assertEqual(self.folder.image.prev_height(), 16)
    #    self.folder.image.manage_http_upload(url=self.app.TiffImage.absolute_url())
    #    # The preview did not change
    #    self.assertEqual(self.folder.image.prev_width(), 25)
    #    self.assertEqual(self.folder.image.prev_height(), 16)


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestExtImageAdd))
    suite.addTest(makeSuite(TestExtImageEdit))
    suite.addTest(makeSuite(TestExtImageCataloging))
    suite.addTest(makeSuite(TestExtImagePreviews))
    return suite

if __name__ == '__main__':
    framework()

