#
# Test preview-related properties
#

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Products.CMFExtFile.tests import CMFExtFileTestCase
from Products.CMFExtFile.config import _get_property
from Products.CMFExtFile.config import *
from Products.ExtFile import ExtImage


class TestGetProperty(CMFExtFileTestCase.CMFExtFileTestCase):

    def afterSetUp(self):
        CMFExtFileTestCase.CMFExtFileTestCase.afterSetUp(self)
        self.portal.extfile_properties.manage_addProperty('tool_prop', 'baz', 'string')

    def testGetProperty(self):
        self.assertEqual(_get_property(self.folder, 'tool_prop'), 'baz')

    def testPropertyDefault(self):
        self.assertEqual(_get_property(self.folder, 'unknown_prop'), None)
        self.assertEqual(_get_property(self.folder, 'unknown_prop', 'peng'), 'peng')


class TestGetPropertyNoTool(CMFExtFileTestCase.CMFExtFileTestCase):

    def afterSetUp(self):
        CMFExtFileTestCase.CMFExtFileTestCase.afterSetUp(self)
        self.portal._delObject('extfile_properties')

    def testGetProperty(self):
        self.assertEqual(_get_property(self.folder, 'tool_prop'), None)

    def testPropertyDefault(self):
        self.assertEqual(_get_property(self.folder, 'unknown_prop'), None)
        self.assertEqual(_get_property(self.folder, 'unknown_prop', 'peng'), 'peng')


class TestExtImageProperties(CMFExtFileTestCase.CMFExtFileTestCase):

    def afterSetUp(self):
        CMFExtFileTestCase.CMFExtFileTestCase.afterSetUp(self)
        self.props = self.portal.extfile_properties

    def testModeOn(self):
        self.props.manage_changeProperties(create_previews=True)
        self.assertEqual(preview_mode(self.folder), ExtImage.GENERATE)
        self.assertEqual(type(preview_mode(self.folder)), type(1))

    def testModeOff(self):
        self.props.manage_changeProperties(create_previews=False)
        self.assertEqual(preview_mode(self.folder), ExtImage.NO_PREVIEW)
        self.assertEqual(type(preview_mode(self.folder)), type(1))

    def testModeDefault(self):
        self.props.manage_delProperties(['create_previews'])
        self.assertEqual(preview_mode(self.folder), ExtImage.GENERATE)
        self.assertEqual(type(preview_mode(self.folder)), type(1))

    def testMaxX(self):
        self.props.manage_changeProperties(preview_maxx=100)
        self.assertEqual(preview_maxx(self.folder), 100)
        self.assertEqual(type(preview_maxx(self.folder)), type(1))

    def testMaxXDefault(self):
        self.props.manage_delProperties(['preview_maxx'])
        self.assertEqual(preview_maxx(self.folder), 350)
        self.assertEqual(type(preview_maxx(self.folder)), type(1))

    def testMaxY(self):
        self.props.manage_changeProperties(preview_maxy=100)
        self.assertEqual(preview_maxy(self.folder), 100)
        self.assertEqual(type(preview_maxy(self.folder)), type(1))

    def testMaxYDefault(self):
        self.props.manage_delProperties(['preview_maxy'])
        self.assertEqual(preview_maxy(self.folder), 350)
        self.assertEqual(type(preview_maxy(self.folder)), type(1))

    def testRatioOn(self):
        self.props.manage_changeProperties(preview_ratio=True)
        self.assertEqual(preview_ratio(self.folder), True)
        self.assertEqual(type(preview_ratio(self.folder)), type(True))

    def testRatioOff(self):
        self.props.manage_changeProperties(preview_ratio=False)
        self.assertEqual(preview_ratio(self.folder), False)
        self.assertEqual(type(preview_ratio(self.folder)), type(True))

    def testRatioDefault(self):
        self.props.manage_delProperties(['preview_ratio'])
        self.assertEqual(preview_ratio(self.folder), True)
        self.assertEqual(type(preview_ratio(self.folder)), type(True))


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestGetProperty))
    suite.addTest(makeSuite(TestGetPropertyNoTool))
    suite.addTest(makeSuite(TestExtImageProperties))
    return suite

if __name__ == '__main__':
    framework()

